# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class SubscribedEventList(ListResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, subscription_sid):
        """
        Initialize the SubscribedEventList

        :param Version version: Version that contains the resource
        :param subscription_sid: Subscription SID.

        :returns: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventList
        :rtype: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventList
        """
        super(SubscribedEventList, self).__init__(version)

        # Path Solution
        self._solution = {'subscription_sid': subscription_sid, }
        self._uri = '/Subscriptions/{subscription_sid}/SubscribedEvents'.format(**self._solution)

    def stream(self, limit=None, page_size=None):
        """
        Streams SubscribedEventInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, limit=None, page_size=None):
        """
        Lists SubscribedEventInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventInstance]
        """
        return list(self.stream(limit=limit, page_size=page_size, ))

    def page(self, page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of SubscribedEventInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of SubscribedEventInstance
        :rtype: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventPage
        """
        data = values.of({'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return SubscribedEventPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of SubscribedEventInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of SubscribedEventInstance
        :rtype: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return SubscribedEventPage(self._version, response, self._solution)

    def create(self, type, schema_version=values.unset):
        """
        Create the SubscribedEventInstance

        :param unicode type: Type of event being subscribed to.
        :param unicode schema_version: The schema version that the subscription should use.

        :returns: The created SubscribedEventInstance
        :rtype: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventInstance
        """
        data = values.of({'Type': type, 'SchemaVersion': schema_version, })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return SubscribedEventInstance(
            self._version,
            payload,
            subscription_sid=self._solution['subscription_sid'],
        )

    def get(self, type):
        """
        Constructs a SubscribedEventContext

        :param type: Type of event being subscribed to.

        :returns: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventContext
        :rtype: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventContext
        """
        return SubscribedEventContext(
            self._version,
            subscription_sid=self._solution['subscription_sid'],
            type=type,
        )

    def __call__(self, type):
        """
        Constructs a SubscribedEventContext

        :param type: Type of event being subscribed to.

        :returns: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventContext
        :rtype: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventContext
        """
        return SubscribedEventContext(
            self._version,
            subscription_sid=self._solution['subscription_sid'],
            type=type,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Events.V1.SubscribedEventList>'


class SubscribedEventPage(Page):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, response, solution):
        """
        Initialize the SubscribedEventPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param subscription_sid: Subscription SID.

        :returns: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventPage
        :rtype: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventPage
        """
        super(SubscribedEventPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of SubscribedEventInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventInstance
        :rtype: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventInstance
        """
        return SubscribedEventInstance(
            self._version,
            payload,
            subscription_sid=self._solution['subscription_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Events.V1.SubscribedEventPage>'


class SubscribedEventContext(InstanceContext):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, subscription_sid, type):
        """
        Initialize the SubscribedEventContext

        :param Version version: Version that contains the resource
        :param subscription_sid: Subscription SID.
        :param type: Type of event being subscribed to.

        :returns: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventContext
        :rtype: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventContext
        """
        super(SubscribedEventContext, self).__init__(version)

        # Path Solution
        self._solution = {'subscription_sid': subscription_sid, 'type': type, }
        self._uri = '/Subscriptions/{subscription_sid}/SubscribedEvents/{type}'.format(**self._solution)

    def fetch(self):
        """
        Fetch the SubscribedEventInstance

        :returns: The fetched SubscribedEventInstance
        :rtype: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return SubscribedEventInstance(
            self._version,
            payload,
            subscription_sid=self._solution['subscription_sid'],
            type=self._solution['type'],
        )

    def update(self, schema_version=values.unset):
        """
        Update the SubscribedEventInstance

        :param unicode schema_version: The schema version that the subscription should use.

        :returns: The updated SubscribedEventInstance
        :rtype: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventInstance
        """
        data = values.of({'SchemaVersion': schema_version, })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return SubscribedEventInstance(
            self._version,
            payload,
            subscription_sid=self._solution['subscription_sid'],
            type=self._solution['type'],
        )

    def delete(self):
        """
        Deletes the SubscribedEventInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Events.V1.SubscribedEventContext {}>'.format(context)


class SubscribedEventInstance(InstanceResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, payload, subscription_sid, type=None):
        """
        Initialize the SubscribedEventInstance

        :returns: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventInstance
        :rtype: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventInstance
        """
        super(SubscribedEventInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'type': payload.get('type'),
            'schema_version': deserialize.integer(payload.get('schema_version')),
            'subscription_sid': payload.get('subscription_sid'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {'subscription_sid': subscription_sid, 'type': type or self._properties['type'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: SubscribedEventContext for this SubscribedEventInstance
        :rtype: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventContext
        """
        if self._context is None:
            self._context = SubscribedEventContext(
                self._version,
                subscription_sid=self._solution['subscription_sid'],
                type=self._solution['type'],
            )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: Account SID.
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def type(self):
        """
        :returns: Type of event being subscribed to.
        :rtype: unicode
        """
        return self._properties['type']

    @property
    def schema_version(self):
        """
        :returns: The schema version that the subscription should use.
        :rtype: unicode
        """
        return self._properties['schema_version']

    @property
    def subscription_sid(self):
        """
        :returns: Subscription SID.
        :rtype: unicode
        """
        return self._properties['subscription_sid']

    @property
    def url(self):
        """
        :returns: The URL of this resource.
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self):
        """
        Fetch the SubscribedEventInstance

        :returns: The fetched SubscribedEventInstance
        :rtype: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventInstance
        """
        return self._proxy.fetch()

    def update(self, schema_version=values.unset):
        """
        Update the SubscribedEventInstance

        :param unicode schema_version: The schema version that the subscription should use.

        :returns: The updated SubscribedEventInstance
        :rtype: twilio.rest.events.v1.subscription.subscribed_event.SubscribedEventInstance
        """
        return self._proxy.update(schema_version=schema_version, )

    def delete(self):
        """
        Deletes the SubscribedEventInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Events.V1.SubscribedEventInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class FleetList(ListResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version):
        """
        Initialize the FleetList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.supersim.v1.fleet.FleetList
        :rtype: twilio.rest.supersim.v1.fleet.FleetList
        """
        super(FleetList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/Fleets'.format(**self._solution)

    def create(self, network_access_profile, unique_name=values.unset,
               data_enabled=values.unset, data_limit=values.unset,
               commands_enabled=values.unset, commands_url=values.unset,
               commands_method=values.unset, sms_commands_enabled=values.unset,
               sms_commands_url=values.unset, sms_commands_method=values.unset):
        """
        Create the FleetInstance

        :param unicode network_access_profile: The SID or unique name of the Network Access Profile of the Fleet
        :param unicode unique_name: An application-defined string that uniquely identifies the resource
        :param bool data_enabled: Defines whether SIMs in the Fleet are capable of using data connectivity
        :param unicode data_limit: The total data usage (download and upload combined) in Megabytes that each Sim resource assigned to the Fleet resource can consume
        :param bool commands_enabled: Defines whether SIMs in the Fleet are capable of sending and receiving machine-to-machine SMS via Commands
        :param unicode commands_url: The URL that will receive a webhook when a Super SIM in the Fleet is used to send an SMS from your device to the Commands number
        :param unicode commands_method: A string representing the HTTP method to use when making a request to `commands_url`
        :param bool sms_commands_enabled: Defines whether SIMs in the Fleet are capable of sending and receiving machine-to-machine SMS via Commands
        :param unicode sms_commands_url: The URL that will receive a webhook when a Super SIM in the Fleet is used to send an SMS from your device to the SMS Commands number
        :param unicode sms_commands_method: A string representing the HTTP method to use when making a request to `sms_commands_url`

        :returns: The created FleetInstance
        :rtype: twilio.rest.supersim.v1.fleet.FleetInstance
        """
        data = values.of({
            'NetworkAccessProfile': network_access_profile,
            'UniqueName': unique_name,
            'DataEnabled': data_enabled,
            'DataLimit': data_limit,
            'CommandsEnabled': commands_enabled,
            'CommandsUrl': commands_url,
            'CommandsMethod': commands_method,
            'SmsCommandsEnabled': sms_commands_enabled,
            'SmsCommandsUrl': sms_commands_url,
            'SmsCommandsMethod': sms_commands_method,
        })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return FleetInstance(self._version, payload, )

    def stream(self, network_access_profile=values.unset, limit=None,
               page_size=None):
        """
        Streams FleetInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param unicode network_access_profile: The SID or unique name of the Network Access Profile of the Fleet
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.supersim.v1.fleet.FleetInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(network_access_profile=network_access_profile, page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, network_access_profile=values.unset, limit=None, page_size=None):
        """
        Lists FleetInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param unicode network_access_profile: The SID or unique name of the Network Access Profile of the Fleet
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.supersim.v1.fleet.FleetInstance]
        """
        return list(self.stream(
            network_access_profile=network_access_profile,
            limit=limit,
            page_size=page_size,
        ))

    def page(self, network_access_profile=values.unset, page_token=values.unset,
             page_number=values.unset, page_size=values.unset):
        """
        Retrieve a single page of FleetInstance records from the API.
        Request is executed immediately

        :param unicode network_access_profile: The SID or unique name of the Network Access Profile of the Fleet
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of FleetInstance
        :rtype: twilio.rest.supersim.v1.fleet.FleetPage
        """
        data = values.of({
            'NetworkAccessProfile': network_access_profile,
            'PageToken': page_token,
            'Page': page_number,
            'PageSize': page_size,
        })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return FleetPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of FleetInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of FleetInstance
        :rtype: twilio.rest.supersim.v1.fleet.FleetPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return FleetPage(self._version, response, self._solution)

    def get(self, sid):
        """
        Constructs a FleetContext

        :param sid: The SID that identifies the resource to fetch

        :returns: twilio.rest.supersim.v1.fleet.FleetContext
        :rtype: twilio.rest.supersim.v1.fleet.FleetContext
        """
        return FleetContext(self._version, sid=sid, )

    def __call__(self, sid):
        """
        Constructs a FleetContext

        :param sid: The SID that identifies the resource to fetch

        :returns: twilio.rest.supersim.v1.fleet.FleetContext
        :rtype: twilio.rest.supersim.v1.fleet.FleetContext
        """
        return FleetContext(self._version, sid=sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Supersim.V1.FleetList>'


class FleetPage(Page):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, response, solution):
        """
        Initialize the FleetPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.supersim.v1.fleet.FleetPage
        :rtype: twilio.rest.supersim.v1.fleet.FleetPage
        """
        super(FleetPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of FleetInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.supersim.v1.fleet.FleetInstance
        :rtype: twilio.rest.supersim.v1.fleet.FleetInstance
        """
        return FleetInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Supersim.V1.FleetPage>'


class FleetContext(InstanceContext):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, sid):
        """
        Initialize the FleetContext

        :param Version version: Version that contains the resource
        :param sid: The SID that identifies the resource to fetch

        :returns: twilio.rest.supersim.v1.fleet.FleetContext
        :rtype: twilio.rest.supersim.v1.fleet.FleetContext
        """
        super(FleetContext, self).__init__(version)

        # Path Solution
        self._solution = {'sid': sid, }
        self._uri = '/Fleets/{sid}'.format(**self._solution)

    def fetch(self):
        """
        Fetch the FleetInstance

        :returns: The fetched FleetInstance
        :rtype: twilio.rest.supersim.v1.fleet.FleetInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return FleetInstance(self._version, payload, sid=self._solution['sid'], )

    def update(self, unique_name=values.unset, network_access_profile=values.unset,
               commands_url=values.unset, commands_method=values.unset,
               sms_commands_url=values.unset, sms_commands_method=values.unset):
        """
        Update the FleetInstance

        :param unicode unique_name: An application-defined string that uniquely identifies the resource
        :param unicode network_access_profile: The SID or unique name of the Network Access Profile of the Fleet
        :param unicode commands_url: The URL that will receive a webhook when a Super SIM in the Fleet is used to send an SMS from your device to the Commands number
        :param unicode commands_method: A string representing the HTTP method to use when making a request to `commands_url`
        :param unicode sms_commands_url: The URL that will receive a webhook when a Super SIM in the Fleet is used to send an SMS from your device to the SMS Commands number
        :param unicode sms_commands_method: A string representing the HTTP method to use when making a request to `sms_commands_url`

        :returns: The updated FleetInstance
        :rtype: twilio.rest.supersim.v1.fleet.FleetInstance
        """
        data = values.of({
            'UniqueName': unique_name,
            'NetworkAccessProfile': network_access_profile,
            'CommandsUrl': commands_url,
            'CommandsMethod': commands_method,
            'SmsCommandsUrl': sms_commands_url,
            'SmsCommandsMethod': sms_commands_method,
        })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return FleetInstance(self._version, payload, sid=self._solution['sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Supersim.V1.FleetContext {}>'.format(context)


class FleetInstance(InstanceResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    class DataMetering(object):
        PAYG = "payg"

    def __init__(self, version, payload, sid=None):
        """
        Initialize the FleetInstance

        :returns: twilio.rest.supersim.v1.fleet.FleetInstance
        :rtype: twilio.rest.supersim.v1.fleet.FleetInstance
        """
        super(FleetInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'sid': payload.get('sid'),
            'unique_name': payload.get('unique_name'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'url': payload.get('url'),
            'data_enabled': payload.get('data_enabled'),
            'data_limit': deserialize.integer(payload.get('data_limit')),
            'data_metering': payload.get('data_metering'),
            'commands_enabled': payload.get('commands_enabled'),
            'commands_url': payload.get('commands_url'),
            'commands_method': payload.get('commands_method'),
            'sms_commands_enabled': payload.get('sms_commands_enabled'),
            'sms_commands_url': payload.get('sms_commands_url'),
            'sms_commands_method': payload.get('sms_commands_method'),
            'network_access_profile_sid': payload.get('network_access_profile_sid'),
        }

        # Context
        self._context = None
        self._solution = {'sid': sid or self._properties['sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: FleetContext for this FleetInstance
        :rtype: twilio.rest.supersim.v1.fleet.FleetContext
        """
        if self._context is None:
            self._context = FleetContext(self._version, sid=self._solution['sid'], )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the resource
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def unique_name(self):
        """
        :returns: An application-defined string that uniquely identifies the resource
        :rtype: unicode
        """
        return self._properties['unique_name']

    @property
    def date_created(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was last updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def url(self):
        """
        :returns: The absolute URL of the Fleet resource
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def data_enabled(self):
        """
        :returns: Defines whether SIMs in the Fleet are capable of using data connectivity
        :rtype: bool
        """
        return self._properties['data_enabled']

    @property
    def data_limit(self):
        """
        :returns: The total data usage (download and upload combined) in Megabytes that each Sim resource assigned to the Fleet resource can consume
        :rtype: unicode
        """
        return self._properties['data_limit']

    @property
    def data_metering(self):
        """
        :returns: The model by which a SIM is metered and billed
        :rtype: FleetInstance.DataMetering
        """
        return self._properties['data_metering']

    @property
    def commands_enabled(self):
        """
        :returns: Defines whether SIMs in the Fleet are capable of sending and receiving machine-to-machine SMS via Commands
        :rtype: bool
        """
        return self._properties['commands_enabled']

    @property
    def commands_url(self):
        """
        :returns: The URL that will receive a webhook when a Super SIM in the Fleet is used to send an SMS from your device to the Commands number
        :rtype: unicode
        """
        return self._properties['commands_url']

    @property
    def commands_method(self):
        """
        :returns: A string representing the HTTP method to use when making a request to `commands_url`
        :rtype: unicode
        """
        return self._properties['commands_method']

    @property
    def sms_commands_enabled(self):
        """
        :returns: Defines whether SIMs in the Fleet are capable of sending and receiving machine-to-machine SMS via Commands
        :rtype: bool
        """
        return self._properties['sms_commands_enabled']

    @property
    def sms_commands_url(self):
        """
        :returns: The URL that will receive a webhook when a Super SIM in the Fleet is used to send an SMS from your device to the SMS Commands number
        :rtype: unicode
        """
        return self._properties['sms_commands_url']

    @property
    def sms_commands_method(self):
        """
        :returns: A string representing the HTTP method to use when making a request to `sms_commands_url`
        :rtype: unicode
        """
        return self._properties['sms_commands_method']

    @property
    def network_access_profile_sid(self):
        """
        :returns: The SID of the Network Access Profile of the Fleet
        :rtype: unicode
        """
        return self._properties['network_access_profile_sid']

    def fetch(self):
        """
        Fetch the FleetInstance

        :returns: The fetched FleetInstance
        :rtype: twilio.rest.supersim.v1.fleet.FleetInstance
        """
        return self._proxy.fetch()

    def update(self, unique_name=values.unset, network_access_profile=values.unset,
               commands_url=values.unset, commands_method=values.unset,
               sms_commands_url=values.unset, sms_commands_method=values.unset):
        """
        Update the FleetInstance

        :param unicode unique_name: An application-defined string that uniquely identifies the resource
        :param unicode network_access_profile: The SID or unique name of the Network Access Profile of the Fleet
        :param unicode commands_url: The URL that will receive a webhook when a Super SIM in the Fleet is used to send an SMS from your device to the Commands number
        :param unicode commands_method: A string representing the HTTP method to use when making a request to `commands_url`
        :param unicode sms_commands_url: The URL that will receive a webhook when a Super SIM in the Fleet is used to send an SMS from your device to the SMS Commands number
        :param unicode sms_commands_method: A string representing the HTTP method to use when making a request to `sms_commands_url`

        :returns: The updated FleetInstance
        :rtype: twilio.rest.supersim.v1.fleet.FleetInstance
        """
        return self._proxy.update(
            unique_name=unique_name,
            network_access_profile=network_access_profile,
            commands_url=commands_url,
            commands_method=commands_method,
            sms_commands_url=sms_commands_url,
            sms_commands_method=sms_commands_method,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Supersim.V1.FleetInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class FieldList(ListResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, assistant_sid, task_sid):
        """
        Initialize the FieldList

        :param Version version: Version that contains the resource
        :param assistant_sid: The unique ID of the parent Assistant.
        :param task_sid: The unique ID of the Task associated with this Field.

        :returns: twilio.rest.preview.understand.assistant.task.field.FieldList
        :rtype: twilio.rest.preview.understand.assistant.task.field.FieldList
        """
        super(FieldList, self).__init__(version)

        # Path Solution
        self._solution = {'assistant_sid': assistant_sid, 'task_sid': task_sid, }
        self._uri = '/Assistants/{assistant_sid}/Tasks/{task_sid}/Fields'.format(**self._solution)

    def stream(self, limit=None, page_size=None):
        """
        Streams FieldInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.preview.understand.assistant.task.field.FieldInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, limit=None, page_size=None):
        """
        Lists FieldInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.preview.understand.assistant.task.field.FieldInstance]
        """
        return list(self.stream(limit=limit, page_size=page_size, ))

    def page(self, page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of FieldInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of FieldInstance
        :rtype: twilio.rest.preview.understand.assistant.task.field.FieldPage
        """
        data = values.of({'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return FieldPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of FieldInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of FieldInstance
        :rtype: twilio.rest.preview.understand.assistant.task.field.FieldPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return FieldPage(self._version, response, self._solution)

    def create(self, field_type, unique_name):
        """
        Create the FieldInstance

        :param unicode field_type: The unique name or sid of the FieldType. It can be any Built-in Field Type or the unique_name or sid of a custom Field Type.
        :param unicode unique_name: A user-provided string that uniquely identifies this resource as an alternative to the sid. Unique up to 64 characters long.

        :returns: The created FieldInstance
        :rtype: twilio.rest.preview.understand.assistant.task.field.FieldInstance
        """
        data = values.of({'FieldType': field_type, 'UniqueName': unique_name, })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return FieldInstance(
            self._version,
            payload,
            assistant_sid=self._solution['assistant_sid'],
            task_sid=self._solution['task_sid'],
        )

    def get(self, sid):
        """
        Constructs a FieldContext

        :param sid: A 34 character string that uniquely identifies this resource.

        :returns: twilio.rest.preview.understand.assistant.task.field.FieldContext
        :rtype: twilio.rest.preview.understand.assistant.task.field.FieldContext
        """
        return FieldContext(
            self._version,
            assistant_sid=self._solution['assistant_sid'],
            task_sid=self._solution['task_sid'],
            sid=sid,
        )

    def __call__(self, sid):
        """
        Constructs a FieldContext

        :param sid: A 34 character string that uniquely identifies this resource.

        :returns: twilio.rest.preview.understand.assistant.task.field.FieldContext
        :rtype: twilio.rest.preview.understand.assistant.task.field.FieldContext
        """
        return FieldContext(
            self._version,
            assistant_sid=self._solution['assistant_sid'],
            task_sid=self._solution['task_sid'],
            sid=sid,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Preview.Understand.FieldList>'


class FieldPage(Page):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, response, solution):
        """
        Initialize the FieldPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param assistant_sid: The unique ID of the parent Assistant.
        :param task_sid: The unique ID of the Task associated with this Field.

        :returns: twilio.rest.preview.understand.assistant.task.field.FieldPage
        :rtype: twilio.rest.preview.understand.assistant.task.field.FieldPage
        """
        super(FieldPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of FieldInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.preview.understand.assistant.task.field.FieldInstance
        :rtype: twilio.rest.preview.understand.assistant.task.field.FieldInstance
        """
        return FieldInstance(
            self._version,
            payload,
            assistant_sid=self._solution['assistant_sid'],
            task_sid=self._solution['task_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Preview.Understand.FieldPage>'


class FieldContext(InstanceContext):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, assistant_sid, task_sid, sid):
        """
        Initialize the FieldContext

        :param Version version: Version that contains the resource
        :param assistant_sid: The unique ID of the Assistant.
        :param task_sid: The unique ID of the Task associated with this Field.
        :param sid: A 34 character string that uniquely identifies this resource.

        :returns: twilio.rest.preview.understand.assistant.task.field.FieldContext
        :rtype: twilio.rest.preview.understand.assistant.task.field.FieldContext
        """
        super(FieldContext, self).__init__(version)

        # Path Solution
        self._solution = {'assistant_sid': assistant_sid, 'task_sid': task_sid, 'sid': sid, }
        self._uri = '/Assistants/{assistant_sid}/Tasks/{task_sid}/Fields/{sid}'.format(**self._solution)

    def fetch(self):
        """
        Fetch the FieldInstance

        :returns: The fetched FieldInstance
        :rtype: twilio.rest.preview.understand.assistant.task.field.FieldInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return FieldInstance(
            self._version,
            payload,
            assistant_sid=self._solution['assistant_sid'],
            task_sid=self._solution['task_sid'],
            sid=self._solution['sid'],
        )

    def delete(self):
        """
        Deletes the FieldInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Preview.Understand.FieldContext {}>'.format(context)


class FieldInstance(InstanceResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, payload, assistant_sid, task_sid, sid=None):
        """
        Initialize the FieldInstance

        :returns: twilio.rest.preview.understand.assistant.task.field.FieldInstance
        :rtype: twilio.rest.preview.understand.assistant.task.field.FieldInstance
        """
        super(FieldInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'field_type': payload.get('field_type'),
            'task_sid': payload.get('task_sid'),
            'assistant_sid': payload.get('assistant_sid'),
            'sid': payload.get('sid'),
            'unique_name': payload.get('unique_name'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {
            'assistant_sid': assistant_sid,
            'task_sid': task_sid,
            'sid': sid or self._properties['sid'],
        }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: FieldContext for this FieldInstance
        :rtype: twilio.rest.preview.understand.assistant.task.field.FieldContext
        """
        if self._context is None:
            self._context = FieldContext(
                self._version,
                assistant_sid=self._solution['assistant_sid'],
                task_sid=self._solution['task_sid'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The unique ID of the Account that created this Field.
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def date_created(self):
        """
        :returns: The date that this resource was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The date that this resource was last updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def field_type(self):
        """
        :returns: The Field Type of this field. It can be any Built-in Field Type or unique_name or the Field Type sid of a custom Field Type.
        :rtype: unicode
        """
        return self._properties['field_type']

    @property
    def task_sid(self):
        """
        :returns: The unique ID of the Task associated with this Field.
        :rtype: unicode
        """
        return self._properties['task_sid']

    @property
    def assistant_sid(self):
        """
        :returns: The unique ID of the parent Assistant.
        :rtype: unicode
        """
        return self._properties['assistant_sid']

    @property
    def sid(self):
        """
        :returns: A 34 character string that uniquely identifies this resource.
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def unique_name(self):
        """
        :returns: A user-provided string that uniquely identifies this resource as an alternative to the sid. Unique up to 64 characters long.
        :rtype: unicode
        """
        return self._properties['unique_name']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self):
        """
        Fetch the FieldInstance

        :returns: The fetched FieldInstance
        :rtype: twilio.rest.preview.understand.assistant.task.field.FieldInstance
        """
        return self._proxy.fetch()

    def delete(self):
        """
        Deletes the FieldInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Preview.Understand.FieldInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class SampleList(ListResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, assistant_sid, task_sid):
        """
        Initialize the SampleList

        :param Version version: Version that contains the resource
        :param assistant_sid: The unique ID of the Assistant.
        :param task_sid: The unique ID of the Task associated with this Sample.

        :returns: twilio.rest.preview.understand.assistant.task.sample.SampleList
        :rtype: twilio.rest.preview.understand.assistant.task.sample.SampleList
        """
        super(SampleList, self).__init__(version)

        # Path Solution
        self._solution = {'assistant_sid': assistant_sid, 'task_sid': task_sid, }
        self._uri = '/Assistants/{assistant_sid}/Tasks/{task_sid}/Samples'.format(**self._solution)

    def stream(self, language=values.unset, limit=None, page_size=None):
        """
        Streams SampleInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param unicode language: An ISO language-country string of the sample.
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.preview.understand.assistant.task.sample.SampleInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(language=language, page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, language=values.unset, limit=None, page_size=None):
        """
        Lists SampleInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param unicode language: An ISO language-country string of the sample.
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.preview.understand.assistant.task.sample.SampleInstance]
        """
        return list(self.stream(language=language, limit=limit, page_size=page_size, ))

    def page(self, language=values.unset, page_token=values.unset,
             page_number=values.unset, page_size=values.unset):
        """
        Retrieve a single page of SampleInstance records from the API.
        Request is executed immediately

        :param unicode language: An ISO language-country string of the sample.
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of SampleInstance
        :rtype: twilio.rest.preview.understand.assistant.task.sample.SamplePage
        """
        data = values.of({
            'Language': language,
            'PageToken': page_token,
            'Page': page_number,
            'PageSize': page_size,
        })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return SamplePage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of SampleInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of SampleInstance
        :rtype: twilio.rest.preview.understand.assistant.task.sample.SamplePage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return SamplePage(self._version, response, self._solution)

    def create(self, language, tagged_text, source_channel=values.unset):
        """
        Create the SampleInstance

        :param unicode language: An ISO language-country string of the sample.
        :param unicode tagged_text: The text example of how end-users may express this task. The sample may contain Field tag blocks.
        :param unicode source_channel: The communication channel the sample was captured. It can be: voice, sms, chat, alexa, google-assistant, or slack. If not included the value will be null

        :returns: The created SampleInstance
        :rtype: twilio.rest.preview.understand.assistant.task.sample.SampleInstance
        """
        data = values.of({'Language': language, 'TaggedText': tagged_text, 'SourceChannel': source_channel, })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return SampleInstance(
            self._version,
            payload,
            assistant_sid=self._solution['assistant_sid'],
            task_sid=self._solution['task_sid'],
        )

    def get(self, sid):
        """
        Constructs a SampleContext

        :param sid: A 34 character string that uniquely identifies this resource.

        :returns: twilio.rest.preview.understand.assistant.task.sample.SampleContext
        :rtype: twilio.rest.preview.understand.assistant.task.sample.SampleContext
        """
        return SampleContext(
            self._version,
            assistant_sid=self._solution['assistant_sid'],
            task_sid=self._solution['task_sid'],
            sid=sid,
        )

    def __call__(self, sid):
        """
        Constructs a SampleContext

        :param sid: A 34 character string that uniquely identifies this resource.

        :returns: twilio.rest.preview.understand.assistant.task.sample.SampleContext
        :rtype: twilio.rest.preview.understand.assistant.task.sample.SampleContext
        """
        return SampleContext(
            self._version,
            assistant_sid=self._solution['assistant_sid'],
            task_sid=self._solution['task_sid'],
            sid=sid,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Preview.Understand.SampleList>'


class SamplePage(Page):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, response, solution):
        """
        Initialize the SamplePage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param assistant_sid: The unique ID of the Assistant.
        :param task_sid: The unique ID of the Task associated with this Sample.

        :returns: twilio.rest.preview.understand.assistant.task.sample.SamplePage
        :rtype: twilio.rest.preview.understand.assistant.task.sample.SamplePage
        """
        super(SamplePage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of SampleInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.preview.understand.assistant.task.sample.SampleInstance
        :rtype: twilio.rest.preview.understand.assistant.task.sample.SampleInstance
        """
        return SampleInstance(
            self._version,
            payload,
            assistant_sid=self._solution['assistant_sid'],
            task_sid=self._solution['task_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Preview.Understand.SamplePage>'


class SampleContext(InstanceContext):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, assistant_sid, task_sid, sid):
        """
        Initialize the SampleContext

        :param Version version: Version that contains the resource
        :param assistant_sid: The unique ID of the Assistant.
        :param task_sid: The unique ID of the Task associated with this Sample.
        :param sid: A 34 character string that uniquely identifies this resource.

        :returns: twilio.rest.preview.understand.assistant.task.sample.SampleContext
        :rtype: twilio.rest.preview.understand.assistant.task.sample.SampleContext
        """
        super(SampleContext, self).__init__(version)

        # Path Solution
        self._solution = {'assistant_sid': assistant_sid, 'task_sid': task_sid, 'sid': sid, }
        self._uri = '/Assistants/{assistant_sid}/Tasks/{task_sid}/Samples/{sid}'.format(**self._solution)

    def fetch(self):
        """
        Fetch the SampleInstance

        :returns: The fetched SampleInstance
        :rtype: twilio.rest.preview.understand.assistant.task.sample.SampleInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return SampleInstance(
            self._version,
            payload,
            assistant_sid=self._solution['assistant_sid'],
            task_sid=self._solution['task_sid'],
            sid=self._solution['sid'],
        )

    def update(self, language=values.unset, tagged_text=values.unset,
               source_channel=values.unset):
        """
        Update the SampleInstance

        :param unicode language: An ISO language-country string of the sample.
        :param unicode tagged_text: The text example of how end-users may express this task. The sample may contain Field tag blocks.
        :param unicode source_channel: The communication channel the sample was captured. It can be: voice, sms, chat, alexa, google-assistant, or slack. If not included the value will be null

        :returns: The updated SampleInstance
        :rtype: twilio.rest.preview.understand.assistant.task.sample.SampleInstance
        """
        data = values.of({'Language': language, 'TaggedText': tagged_text, 'SourceChannel': source_channel, })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return SampleInstance(
            self._version,
            payload,
            assistant_sid=self._solution['assistant_sid'],
            task_sid=self._solution['task_sid'],
            sid=self._solution['sid'],
        )

    def delete(self):
        """
        Deletes the SampleInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Preview.Understand.SampleContext {}>'.format(context)


class SampleInstance(InstanceResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, payload, assistant_sid, task_sid, sid=None):
        """
        Initialize the SampleInstance

        :returns: twilio.rest.preview.understand.assistant.task.sample.SampleInstance
        :rtype: twilio.rest.preview.understand.assistant.task.sample.SampleInstance
        """
        super(SampleInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'task_sid': payload.get('task_sid'),
            'language': payload.get('language'),
            'assistant_sid': payload.get('assistant_sid'),
            'sid': payload.get('sid'),
            'tagged_text': payload.get('tagged_text'),
            'url': payload.get('url'),
            'source_channel': payload.get('source_channel'),
        }

        # Context
        self._context = None
        self._solution = {
            'assistant_sid': assistant_sid,
            'task_sid': task_sid,
            'sid': sid or self._properties['sid'],
        }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: SampleContext for this SampleInstance
        :rtype: twilio.rest.preview.understand.assistant.task.sample.SampleContext
        """
        if self._context is None:
            self._context = SampleContext(
                self._version,
                assistant_sid=self._solution['assistant_sid'],
                task_sid=self._solution['task_sid'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The unique ID of the Account that created this Sample.
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def date_created(self):
        """
        :returns: The date that this resource was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The date that this resource was last updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def task_sid(self):
        """
        :returns: The unique ID of the Task associated with this Sample.
        :rtype: unicode
        """
        return self._properties['task_sid']

    @property
    def language(self):
        """
        :returns: An ISO language-country string of the sample.
        :rtype: unicode
        """
        return self._properties['language']

    @property
    def assistant_sid(self):
        """
        :returns: The unique ID of the Assistant.
        :rtype: unicode
        """
        return self._properties['assistant_sid']

    @property
    def sid(self):
        """
        :returns: A 34 character string that uniquely identifies this resource.
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def tagged_text(self):
        """
        :returns: The text example of how end-users may express this task. The sample may contain Field tag blocks.
        :rtype: unicode
        """
        return self._properties['tagged_text']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def source_channel(self):
        """
        :returns: The communication channel the sample was captured. It can be: voice, sms, chat, alexa, google-assistant, or slack. If not included the value will be null
        :rtype: unicode
        """
        return self._properties['source_channel']

    def fetch(self):
        """
        Fetch the SampleInstance

        :returns: The fetched SampleInstance
        :rtype: twilio.rest.preview.understand.assistant.task.sample.SampleInstance
        """
        return self._proxy.fetch()

    def update(self, language=values.unset, tagged_text=values.unset,
               source_channel=values.unset):
        """
        Update the SampleInstance

        :param unicode language: An ISO language-country string of the sample.
        :param unicode tagged_text: The text example of how end-users may express this task. The sample may contain Field tag blocks.
        :param unicode source_channel: The communication channel the sample was captured. It can be: voice, sms, chat, alexa, google-assistant, or slack. If not included the value will be null

        :returns: The updated SampleInstance
        :rtype: twilio.rest.preview.understand.assistant.task.sample.SampleInstance
        """
        return self._proxy.update(language=language, tagged_text=tagged_text, source_channel=source_channel, )

    def delete(self):
        """
        Deletes the SampleInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Preview.Understand.SampleInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.version import Version
from twilio.rest.preview.deployed_devices.fleet import FleetList


class DeployedDevices(Version):

    def __init__(self, domain):
        """
        Initialize the DeployedDevices version of Preview

        :returns: DeployedDevices version of Preview
        :rtype: twilio.rest.preview.deployed_devices.DeployedDevices.DeployedDevices
        """
        super(DeployedDevices, self).__init__(domain)
        self.version = 'DeployedDevices'
        self._fleets = None

    @property
    def fleets(self):
        """
        :rtype: twilio.rest.preview.deployed_devices.fleet.FleetList
        """
        if self._fleets is None:
            self._fleets = FleetList(self)
        return self._fleets

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Preview.DeployedDevices>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.version import Version
from twilio.rest.preview.hosted_numbers.authorization_document import AuthorizationDocumentList
from twilio.rest.preview.hosted_numbers.hosted_number_order import HostedNumberOrderList


class HostedNumbers(Version):

    def __init__(self, domain):
        """
        Initialize the HostedNumbers version of Preview

        :returns: HostedNumbers version of Preview
        :rtype: twilio.rest.preview.hosted_numbers.HostedNumbers.HostedNumbers
        """
        super(HostedNumbers, self).__init__(domain)
        self.version = 'HostedNumbers'
        self._authorization_documents = None
        self._hosted_number_orders = None

    @property
    def authorization_documents(self):
        """
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentList
        """
        if self._authorization_documents is None:
            self._authorization_documents = AuthorizationDocumentList(self)
        return self._authorization_documents

    @property
    def hosted_number_orders(self):
        """
        :rtype: twilio.rest.preview.hosted_numbers.hosted_number_order.HostedNumberOrderList
        """
        if self._hosted_number_orders is None:
            self._hosted_number_orders = HostedNumberOrderList(self)
        return self._hosted_number_orders

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Preview.HostedNumbers>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class TemplateList(ListResource):

    def __init__(self, version):
        """
        Initialize the TemplateList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.verify.v2.template.TemplateList
        :rtype: twilio.rest.verify.v2.template.TemplateList
        """
        super(TemplateList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/Templates'.format(**self._solution)

    def stream(self, friendly_name=values.unset, limit=None, page_size=None):
        """
        Streams TemplateInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param unicode friendly_name: Filter templates using friendly name
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.verify.v2.template.TemplateInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(friendly_name=friendly_name, page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, friendly_name=values.unset, limit=None, page_size=None):
        """
        Lists TemplateInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param unicode friendly_name: Filter templates using friendly name
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.verify.v2.template.TemplateInstance]
        """
        return list(self.stream(friendly_name=friendly_name, limit=limit, page_size=page_size, ))

    def page(self, friendly_name=values.unset, page_token=values.unset,
             page_number=values.unset, page_size=values.unset):
        """
        Retrieve a single page of TemplateInstance records from the API.
        Request is executed immediately

        :param unicode friendly_name: Filter templates using friendly name
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of TemplateInstance
        :rtype: twilio.rest.verify.v2.template.TemplatePage
        """
        data = values.of({
            'FriendlyName': friendly_name,
            'PageToken': page_token,
            'Page': page_number,
            'PageSize': page_size,
        })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return TemplatePage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of TemplateInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of TemplateInstance
        :rtype: twilio.rest.verify.v2.template.TemplatePage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return TemplatePage(self._version, response, self._solution)

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Verify.V2.TemplateList>'


class TemplatePage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the TemplatePage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.verify.v2.template.TemplatePage
        :rtype: twilio.rest.verify.v2.template.TemplatePage
        """
        super(TemplatePage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of TemplateInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.verify.v2.template.TemplateInstance
        :rtype: twilio.rest.verify.v2.template.TemplateInstance
        """
        return TemplateInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Verify.V2.TemplatePage>'


class TemplateInstance(InstanceResource):

    def __init__(self, version, payload):
        """
        Initialize the TemplateInstance

        :returns: twilio.rest.verify.v2.template.TemplateInstance
        :rtype: twilio.rest.verify.v2.template.TemplateInstance
        """
        super(TemplateInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'account_sid': payload.get('account_sid'),
            'friendly_name': payload.get('friendly_name'),
            'translations': payload.get('translations'),
        }

        # Context
        self._context = None
        self._solution = {}

    @property
    def sid(self):
        """
        :returns: A string that uniquely identifies this Template
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def account_sid(self):
        """
        :returns: Account Sid
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def friendly_name(self):
        """
        :returns: A string to describe the verification template
        :rtype: unicode
        """
        return self._properties['friendly_name']

    @property
    def translations(self):
        """
        :returns: Ojbect with the template translations.
        :rtype: dict
        """
        return self._properties['translations']

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Verify.V2.TemplateInstance>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.version import Version
from twilio.rest.voice.v1.byoc_trunk import ByocTrunkList
from twilio.rest.voice.v1.connection_policy import ConnectionPolicyList
from twilio.rest.voice.v1.dialing_permissions import DialingPermissionsList
from twilio.rest.voice.v1.ip_record import IpRecordList
from twilio.rest.voice.v1.source_ip_mapping import SourceIpMappingList


class V1(Version):

    def __init__(self, domain):
        """
        Initialize the V1 version of Voice

        :returns: V1 version of Voice
        :rtype: twilio.rest.voice.v1.V1.V1
        """
        super(V1, self).__init__(domain)
        self.version = 'v1'
        self._byoc_trunks = None
        self._connection_policies = None
        self._dialing_permissions = None
        self._ip_records = None
        self._source_ip_mappings = None

    @property
    def byoc_trunks(self):
        """
        :rtype: twilio.rest.voice.v1.byoc_trunk.ByocTrunkList
        """
        if self._byoc_trunks is None:
            self._byoc_trunks = ByocTrunkList(self)
        return self._byoc_trunks

    @property
    def connection_policies(self):
        """
        :rtype: twilio.rest.voice.v1.connection_policy.ConnectionPolicyList
        """
        if self._connection_policies is None:
            self._connection_policies = ConnectionPolicyList(self)
        return self._connection_policies

    @property
    def dialing_permissions(self):
        """
        :rtype: twilio.rest.voice.v1.dialing_permissions.DialingPermissionsList
        """
        if self._dialing_permissions is None:
            self._dialing_permissions = DialingPermissionsList(self)
        return self._dialing_permissions

    @property
    def ip_records(self):
        """
        :rtype: twilio.rest.voice.v1.ip_record.IpRecordList
        """
        if self._ip_records is None:
            self._ip_records = IpRecordList(self)
        return self._ip_records

    @property
    def source_ip_mappings(self):
        """
        :rtype: twilio.rest.voice.v1.source_ip_mapping.SourceIpMappingList
        """
        if self._source_ip_mappings is None:
            self._source_ip_mappings = SourceIpMappingList(self)
        return self._source_ip_mappings

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Voice.V1>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class DeviceList(ListResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, fleet_sid):
        """
        Initialize the DeviceList

        :param Version version: Version that contains the resource
        :param fleet_sid: The unique identifier of the Fleet.

        :returns: twilio.rest.preview.deployed_devices.fleet.device.DeviceList
        :rtype: twilio.rest.preview.deployed_devices.fleet.device.DeviceList
        """
        super(DeviceList, self).__init__(version)

        # Path Solution
        self._solution = {'fleet_sid': fleet_sid, }
        self._uri = '/Fleets/{fleet_sid}/Devices'.format(**self._solution)

    def create(self, unique_name=values.unset, friendly_name=values.unset,
               identity=values.unset, deployment_sid=values.unset,
               enabled=values.unset):
        """
        Create the DeviceInstance

        :param unicode unique_name: A unique, addressable name of this Device.
        :param unicode friendly_name: A human readable description for this Device.
        :param unicode identity: An identifier of the Device user.
        :param unicode deployment_sid: The unique SID of the Deployment group.
        :param bool enabled: The enabled

        :returns: The created DeviceInstance
        :rtype: twilio.rest.preview.deployed_devices.fleet.device.DeviceInstance
        """
        data = values.of({
            'UniqueName': unique_name,
            'FriendlyName': friendly_name,
            'Identity': identity,
            'DeploymentSid': deployment_sid,
            'Enabled': enabled,
        })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return DeviceInstance(self._version, payload, fleet_sid=self._solution['fleet_sid'], )

    def stream(self, deployment_sid=values.unset, limit=None, page_size=None):
        """
        Streams DeviceInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param unicode deployment_sid: Find all Devices grouped under the specified Deployment.
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.preview.deployed_devices.fleet.device.DeviceInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(deployment_sid=deployment_sid, page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, deployment_sid=values.unset, limit=None, page_size=None):
        """
        Lists DeviceInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param unicode deployment_sid: Find all Devices grouped under the specified Deployment.
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.preview.deployed_devices.fleet.device.DeviceInstance]
        """
        return list(self.stream(deployment_sid=deployment_sid, limit=limit, page_size=page_size, ))

    def page(self, deployment_sid=values.unset, page_token=values.unset,
             page_number=values.unset, page_size=values.unset):
        """
        Retrieve a single page of DeviceInstance records from the API.
        Request is executed immediately

        :param unicode deployment_sid: Find all Devices grouped under the specified Deployment.
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of DeviceInstance
        :rtype: twilio.rest.preview.deployed_devices.fleet.device.DevicePage
        """
        data = values.of({
            'DeploymentSid': deployment_sid,
            'PageToken': page_token,
            'Page': page_number,
            'PageSize': page_size,
        })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return DevicePage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of DeviceInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of DeviceInstance
        :rtype: twilio.rest.preview.deployed_devices.fleet.device.DevicePage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return DevicePage(self._version, response, self._solution)

    def get(self, sid):
        """
        Constructs a DeviceContext

        :param sid: A string that uniquely identifies the Device.

        :returns: twilio.rest.preview.deployed_devices.fleet.device.DeviceContext
        :rtype: twilio.rest.preview.deployed_devices.fleet.device.DeviceContext
        """
        return DeviceContext(self._version, fleet_sid=self._solution['fleet_sid'], sid=sid, )

    def __call__(self, sid):
        """
        Constructs a DeviceContext

        :param sid: A string that uniquely identifies the Device.

        :returns: twilio.rest.preview.deployed_devices.fleet.device.DeviceContext
        :rtype: twilio.rest.preview.deployed_devices.fleet.device.DeviceContext
        """
        return DeviceContext(self._version, fleet_sid=self._solution['fleet_sid'], sid=sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Preview.DeployedDevices.DeviceList>'


class DevicePage(Page):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, response, solution):
        """
        Initialize the DevicePage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param fleet_sid: The unique identifier of the Fleet.

        :returns: twilio.rest.preview.deployed_devices.fleet.device.DevicePage
        :rtype: twilio.rest.preview.deployed_devices.fleet.device.DevicePage
        """
        super(DevicePage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of DeviceInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.preview.deployed_devices.fleet.device.DeviceInstance
        :rtype: twilio.rest.preview.deployed_devices.fleet.device.DeviceInstance
        """
        return DeviceInstance(self._version, payload, fleet_sid=self._solution['fleet_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Preview.DeployedDevices.DevicePage>'


class DeviceContext(InstanceContext):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, fleet_sid, sid):
        """
        Initialize the DeviceContext

        :param Version version: Version that contains the resource
        :param fleet_sid: The fleet_sid
        :param sid: A string that uniquely identifies the Device.

        :returns: twilio.rest.preview.deployed_devices.fleet.device.DeviceContext
        :rtype: twilio.rest.preview.deployed_devices.fleet.device.DeviceContext
        """
        super(DeviceContext, self).__init__(version)

        # Path Solution
        self._solution = {'fleet_sid': fleet_sid, 'sid': sid, }
        self._uri = '/Fleets/{fleet_sid}/Devices/{sid}'.format(**self._solution)

    def fetch(self):
        """
        Fetch the DeviceInstance

        :returns: The fetched DeviceInstance
        :rtype: twilio.rest.preview.deployed_devices.fleet.device.DeviceInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return DeviceInstance(
            self._version,
            payload,
            fleet_sid=self._solution['fleet_sid'],
            sid=self._solution['sid'],
        )

    def delete(self):
        """
        Deletes the DeviceInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def update(self, friendly_name=values.unset, identity=values.unset,
               deployment_sid=values.unset, enabled=values.unset):
        """
        Update the DeviceInstance

        :param unicode friendly_name: A human readable description for this Device.
        :param unicode identity: An identifier of the Device user.
        :param unicode deployment_sid: The unique SID of the Deployment group.
        :param bool enabled: The enabled

        :returns: The updated DeviceInstance
        :rtype: twilio.rest.preview.deployed_devices.fleet.device.DeviceInstance
        """
        data = values.of({
            'FriendlyName': friendly_name,
            'Identity': identity,
            'DeploymentSid': deployment_sid,
            'Enabled': enabled,
        })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return DeviceInstance(
            self._version,
            payload,
            fleet_sid=self._solution['fleet_sid'],
            sid=self._solution['sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Preview.DeployedDevices.DeviceContext {}>'.format(context)


class DeviceInstance(InstanceResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, payload, fleet_sid, sid=None):
        """
        Initialize the DeviceInstance

        :returns: twilio.rest.preview.deployed_devices.fleet.device.DeviceInstance
        :rtype: twilio.rest.preview.deployed_devices.fleet.device.DeviceInstance
        """
        super(DeviceInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'url': payload.get('url'),
            'unique_name': payload.get('unique_name'),
            'friendly_name': payload.get('friendly_name'),
            'fleet_sid': payload.get('fleet_sid'),
            'enabled': payload.get('enabled'),
            'account_sid': payload.get('account_sid'),
            'identity': payload.get('identity'),
            'deployment_sid': payload.get('deployment_sid'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'date_authenticated': deserialize.iso8601_datetime(payload.get('date_authenticated')),
        }

        # Context
        self._context = None
        self._solution = {'fleet_sid': fleet_sid, 'sid': sid or self._properties['sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: DeviceContext for this DeviceInstance
        :rtype: twilio.rest.preview.deployed_devices.fleet.device.DeviceContext
        """
        if self._context is None:
            self._context = DeviceContext(
                self._version,
                fleet_sid=self._solution['fleet_sid'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def sid(self):
        """
        :returns: A string that uniquely identifies this Device.
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def url(self):
        """
        :returns: URL of this Device.
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def unique_name(self):
        """
        :returns: A unique, addressable name of this Device.
        :rtype: unicode
        """
        return self._properties['unique_name']

    @property
    def friendly_name(self):
        """
        :returns: A human readable description for this Device
        :rtype: unicode
        """
        return self._properties['friendly_name']

    @property
    def fleet_sid(self):
        """
        :returns: The unique identifier of the Fleet.
        :rtype: unicode
        """
        return self._properties['fleet_sid']

    @property
    def enabled(self):
        """
        :returns: Device enabled flag.
        :rtype: bool
        """
        return self._properties['enabled']

    @property
    def account_sid(self):
        """
        :returns: The unique SID that identifies this Account.
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def identity(self):
        """
        :returns: An identifier of the Device user.
        :rtype: unicode
        """
        return self._properties['identity']

    @property
    def deployment_sid(self):
        """
        :returns: The unique SID of the Deployment group.
        :rtype: unicode
        """
        return self._properties['deployment_sid']

    @property
    def date_created(self):
        """
        :returns: The date this Device was created.
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The date this Device was updated.
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def date_authenticated(self):
        """
        :returns: The date this Device was authenticated.
        :rtype: datetime
        """
        return self._properties['date_authenticated']

    def fetch(self):
        """
        Fetch the DeviceInstance

        :returns: The fetched DeviceInstance
        :rtype: twilio.rest.preview.deployed_devices.fleet.device.DeviceInstance
        """
        return self._proxy.fetch()

    def delete(self):
        """
        Deletes the DeviceInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def update(self, friendly_name=values.unset, identity=values.unset,
               deployment_sid=values.unset, enabled=values.unset):
        """
        Update the DeviceInstance

        :param unicode friendly_name: A human readable description for this Device.
        :param unicode identity: An identifier of the Device user.
        :param unicode deployment_sid: The unique SID of the Deployment group.
        :param bool enabled: The enabled

        :returns: The updated DeviceInstance
        :rtype: twilio.rest.preview.deployed_devices.fleet.device.DeviceInstance
        """
        return self._proxy.update(
            friendly_name=friendly_name,
            identity=identity,
            deployment_sid=deployment_sid,
            enabled=enabled,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Preview.DeployedDevices.DeviceInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class CallSummariesList(ListResource):

    def __init__(self, version):
        """
        Initialize the CallSummariesList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.insights.v1.call_summaries.CallSummariesList
        :rtype: twilio.rest.insights.v1.call_summaries.CallSummariesList
        """
        super(CallSummariesList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/Voice/Summaries'.format(**self._solution)

    def stream(self, from_=values.unset, to=values.unset, from_carrier=values.unset,
               to_carrier=values.unset, from_country_code=values.unset,
               to_country_code=values.unset, branded=values.unset,
               verified_caller=values.unset, has_tag=values.unset,
               start_time=values.unset, end_time=values.unset,
               call_type=values.unset, call_state=values.unset,
               direction=values.unset, processing_state=values.unset,
               sort_by=values.unset, subaccount=values.unset,
               abnormal_session=values.unset, limit=None, page_size=None):
        """
        Streams CallSummariesInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param unicode from_: The from
        :param unicode to: The to
        :param unicode from_carrier: The from_carrier
        :param unicode to_carrier: The to_carrier
        :param unicode from_country_code: The from_country_code
        :param unicode to_country_code: The to_country_code
        :param bool branded: The branded
        :param bool verified_caller: The verified_caller
        :param bool has_tag: The has_tag
        :param unicode start_time: The start_time
        :param unicode end_time: The end_time
        :param unicode call_type: The call_type
        :param unicode call_state: The call_state
        :param unicode direction: The direction
        :param CallSummariesInstance.ProcessingStateRequest processing_state: The processing_state
        :param CallSummariesInstance.SortBy sort_by: The sort_by
        :param unicode subaccount: The subaccount
        :param bool abnormal_session: The abnormal_session
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.insights.v1.call_summaries.CallSummariesInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(
            from_=from_,
            to=to,
            from_carrier=from_carrier,
            to_carrier=to_carrier,
            from_country_code=from_country_code,
            to_country_code=to_country_code,
            branded=branded,
            verified_caller=verified_caller,
            has_tag=has_tag,
            start_time=start_time,
            end_time=end_time,
            call_type=call_type,
            call_state=call_state,
            direction=direction,
            processing_state=processing_state,
            sort_by=sort_by,
            subaccount=subaccount,
            abnormal_session=abnormal_session,
            page_size=limits['page_size'],
        )

        return self._version.stream(page, limits['limit'])

    def list(self, from_=values.unset, to=values.unset, from_carrier=values.unset,
             to_carrier=values.unset, from_country_code=values.unset,
             to_country_code=values.unset, branded=values.unset,
             verified_caller=values.unset, has_tag=values.unset,
             start_time=values.unset, end_time=values.unset, call_type=values.unset,
             call_state=values.unset, direction=values.unset,
             processing_state=values.unset, sort_by=values.unset,
             subaccount=values.unset, abnormal_session=values.unset, limit=None,
             page_size=None):
        """
        Lists CallSummariesInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param unicode from_: The from
        :param unicode to: The to
        :param unicode from_carrier: The from_carrier
        :param unicode to_carrier: The to_carrier
        :param unicode from_country_code: The from_country_code
        :param unicode to_country_code: The to_country_code
        :param bool branded: The branded
        :param bool verified_caller: The verified_caller
        :param bool has_tag: The has_tag
        :param unicode start_time: The start_time
        :param unicode end_time: The end_time
        :param unicode call_type: The call_type
        :param unicode call_state: The call_state
        :param unicode direction: The direction
        :param CallSummariesInstance.ProcessingStateRequest processing_state: The processing_state
        :param CallSummariesInstance.SortBy sort_by: The sort_by
        :param unicode subaccount: The subaccount
        :param bool abnormal_session: The abnormal_session
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.insights.v1.call_summaries.CallSummariesInstance]
        """
        return list(self.stream(
            from_=from_,
            to=to,
            from_carrier=from_carrier,
            to_carrier=to_carrier,
            from_country_code=from_country_code,
            to_country_code=to_country_code,
            branded=branded,
            verified_caller=verified_caller,
            has_tag=has_tag,
            start_time=start_time,
            end_time=end_time,
            call_type=call_type,
            call_state=call_state,
            direction=direction,
            processing_state=processing_state,
            sort_by=sort_by,
            subaccount=subaccount,
            abnormal_session=abnormal_session,
            limit=limit,
            page_size=page_size,
        ))

    def page(self, from_=values.unset, to=values.unset, from_carrier=values.unset,
             to_carrier=values.unset, from_country_code=values.unset,
             to_country_code=values.unset, branded=values.unset,
             verified_caller=values.unset, has_tag=values.unset,
             start_time=values.unset, end_time=values.unset, call_type=values.unset,
             call_state=values.unset, direction=values.unset,
             processing_state=values.unset, sort_by=values.unset,
             subaccount=values.unset, abnormal_session=values.unset,
             page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of CallSummariesInstance records from the API.
        Request is executed immediately

        :param unicode from_: The from
        :param unicode to: The to
        :param unicode from_carrier: The from_carrier
        :param unicode to_carrier: The to_carrier
        :param unicode from_country_code: The from_country_code
        :param unicode to_country_code: The to_country_code
        :param bool branded: The branded
        :param bool verified_caller: The verified_caller
        :param bool has_tag: The has_tag
        :param unicode start_time: The start_time
        :param unicode end_time: The end_time
        :param unicode call_type: The call_type
        :param unicode call_state: The call_state
        :param unicode direction: The direction
        :param CallSummariesInstance.ProcessingStateRequest processing_state: The processing_state
        :param CallSummariesInstance.SortBy sort_by: The sort_by
        :param unicode subaccount: The subaccount
        :param bool abnormal_session: The abnormal_session
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of CallSummariesInstance
        :rtype: twilio.rest.insights.v1.call_summaries.CallSummariesPage
        """
        data = values.of({
            'From': from_,
            'To': to,
            'FromCarrier': from_carrier,
            'ToCarrier': to_carrier,
            'FromCountryCode': from_country_code,
            'ToCountryCode': to_country_code,
            'Branded': branded,
            'VerifiedCaller': verified_caller,
            'HasTag': has_tag,
            'StartTime': start_time,
            'EndTime': end_time,
            'CallType': call_type,
            'CallState': call_state,
            'Direction': direction,
            'ProcessingState': processing_state,
            'SortBy': sort_by,
            'Subaccount': subaccount,
            'AbnormalSession': abnormal_session,
            'PageToken': page_token,
            'Page': page_number,
            'PageSize': page_size,
        })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return CallSummariesPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of CallSummariesInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of CallSummariesInstance
        :rtype: twilio.rest.insights.v1.call_summaries.CallSummariesPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return CallSummariesPage(self._version, response, self._solution)

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Insights.V1.CallSummariesList>'


class CallSummariesPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the CallSummariesPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.insights.v1.call_summaries.CallSummariesPage
        :rtype: twilio.rest.insights.v1.call_summaries.CallSummariesPage
        """
        super(CallSummariesPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of CallSummariesInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.insights.v1.call_summaries.CallSummariesInstance
        :rtype: twilio.rest.insights.v1.call_summaries.CallSummariesInstance
        """
        return CallSummariesInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Insights.V1.CallSummariesPage>'


class CallSummariesInstance(InstanceResource):

    class CallType(object):
        CARRIER = "carrier"
        SIP = "sip"
        TRUNKING = "trunking"
        CLIENT = "client"

    class CallState(object):
        RINGING = "ringing"
        COMPLETED = "completed"
        BUSY = "busy"
        FAIL = "fail"
        NOANSWER = "noanswer"
        CANCELED = "canceled"
        ANSWERED = "answered"
        UNDIALED = "undialed"

    class ProcessingState(object):
        COMPLETE = "complete"
        PARTIAL = "partial"

    class CallDirection(object):
        OUTBOUND_API = "outbound_api"
        OUTBOUND_DIAL = "outbound_dial"
        INBOUND = "inbound"
        TRUNKING_ORIGINATING = "trunking_originating"
        TRUNKING_TERMINATING = "trunking_terminating"

    class SortBy(object):
        START_TIME = "start_time"
        END_TIME = "end_time"

    class ProcessingStateRequest(object):
        COMPLETED = "completed"
        STARTED = "started"
        PARTIAL = "partial"
        ALL = "all"

    def __init__(self, version, payload):
        """
        Initialize the CallSummariesInstance

        :returns: twilio.rest.insights.v1.call_summaries.CallSummariesInstance
        :rtype: twilio.rest.insights.v1.call_summaries.CallSummariesInstance
        """
        super(CallSummariesInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'call_sid': payload.get('call_sid'),
            'call_type': payload.get('call_type'),
            'call_state': payload.get('call_state'),
            'processing_state': payload.get('processing_state'),
            'created_time': deserialize.iso8601_datetime(payload.get('created_time')),
            'start_time': deserialize.iso8601_datetime(payload.get('start_time')),
            'end_time': deserialize.iso8601_datetime(payload.get('end_time')),
            'duration': deserialize.integer(payload.get('duration')),
            'connect_duration': deserialize.integer(payload.get('connect_duration')),
            'from_': payload.get('from'),
            'to': payload.get('to'),
            'carrier_edge': payload.get('carrier_edge'),
            'client_edge': payload.get('client_edge'),
            'sdk_edge': payload.get('sdk_edge'),
            'sip_edge': payload.get('sip_edge'),
            'tags': payload.get('tags'),
            'url': payload.get('url'),
            'attributes': payload.get('attributes'),
            'properties': payload.get('properties'),
            'trust': payload.get('trust'),
        }

        # Context
        self._context = None
        self._solution = {}

    @property
    def account_sid(self):
        """
        :returns: The account_sid
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def call_sid(self):
        """
        :returns: The call_sid
        :rtype: unicode
        """
        return self._properties['call_sid']

    @property
    def call_type(self):
        """
        :returns: The call_type
        :rtype: CallSummariesInstance.CallType
        """
        return self._properties['call_type']

    @property
    def call_state(self):
        """
        :returns: The call_state
        :rtype: CallSummariesInstance.CallState
        """
        return self._properties['call_state']

    @property
    def processing_state(self):
        """
        :returns: The processing_state
        :rtype: CallSummariesInstance.ProcessingState
        """
        return self._properties['processing_state']

    @property
    def created_time(self):
        """
        :returns: The created_time
        :rtype: datetime
        """
        return self._properties['created_time']

    @property
    def start_time(self):
        """
        :returns: The start_time
        :rtype: datetime
        """
        return self._properties['start_time']

    @property
    def end_time(self):
        """
        :returns: The end_time
        :rtype: datetime
        """
        return self._properties['end_time']

    @property
    def duration(self):
        """
        :returns: The duration
        :rtype: unicode
        """
        return self._properties['duration']

    @property
    def connect_duration(self):
        """
        :returns: The connect_duration
        :rtype: unicode
        """
        return self._properties['connect_duration']

    @property
    def from_(self):
        """
        :returns: The from
        :rtype: dict
        """
        return self._properties['from_']

    @property
    def to(self):
        """
        :returns: The to
        :rtype: dict
        """
        return self._properties['to']

    @property
    def carrier_edge(self):
        """
        :returns: The carrier_edge
        :rtype: dict
        """
        return self._properties['carrier_edge']

    @property
    def client_edge(self):
        """
        :returns: The client_edge
        :rtype: dict
        """
        return self._properties['client_edge']

    @property
    def sdk_edge(self):
        """
        :returns: The sdk_edge
        :rtype: dict
        """
        return self._properties['sdk_edge']

    @property
    def sip_edge(self):
        """
        :returns: The sip_edge
        :rtype: dict
        """
        return self._properties['sip_edge']

    @property
    def tags(self):
        """
        :returns: The tags
        :rtype: list[unicode]
        """
        return self._properties['tags']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def attributes(self):
        """
        :returns: The attributes
        :rtype: dict
        """
        return self._properties['attributes']

    @property
    def properties(self):
        """
        :returns: The properties
        :rtype: dict
        """
        return self._properties['properties']

    @property
    def trust(self):
        """
        :returns: The trust
        :rtype: dict
        """
        return self._properties['trust']

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Insights.V1.CallSummariesInstance>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class BillingPeriodList(ListResource):

    def __init__(self, version, sim_sid):
        """
        Initialize the BillingPeriodList

        :param Version version: Version that contains the resource
        :param sim_sid: The SID of the Super SIM the Billing Period belongs to

        :returns: twilio.rest.supersim.v1.sim.billing_period.BillingPeriodList
        :rtype: twilio.rest.supersim.v1.sim.billing_period.BillingPeriodList
        """
        super(BillingPeriodList, self).__init__(version)

        # Path Solution
        self._solution = {'sim_sid': sim_sid, }
        self._uri = '/Sims/{sim_sid}/BillingPeriods'.format(**self._solution)

    def stream(self, limit=None, page_size=None):
        """
        Streams BillingPeriodInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.supersim.v1.sim.billing_period.BillingPeriodInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, limit=None, page_size=None):
        """
        Lists BillingPeriodInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.supersim.v1.sim.billing_period.BillingPeriodInstance]
        """
        return list(self.stream(limit=limit, page_size=page_size, ))

    def page(self, page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of BillingPeriodInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of BillingPeriodInstance
        :rtype: twilio.rest.supersim.v1.sim.billing_period.BillingPeriodPage
        """
        data = values.of({'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return BillingPeriodPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of BillingPeriodInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of BillingPeriodInstance
        :rtype: twilio.rest.supersim.v1.sim.billing_period.BillingPeriodPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return BillingPeriodPage(self._version, response, self._solution)

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Supersim.V1.BillingPeriodList>'


class BillingPeriodPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the BillingPeriodPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param sim_sid: The SID of the Super SIM the Billing Period belongs to

        :returns: twilio.rest.supersim.v1.sim.billing_period.BillingPeriodPage
        :rtype: twilio.rest.supersim.v1.sim.billing_period.BillingPeriodPage
        """
        super(BillingPeriodPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of BillingPeriodInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.supersim.v1.sim.billing_period.BillingPeriodInstance
        :rtype: twilio.rest.supersim.v1.sim.billing_period.BillingPeriodInstance
        """
        return BillingPeriodInstance(self._version, payload, sim_sid=self._solution['sim_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Supersim.V1.BillingPeriodPage>'


class BillingPeriodInstance(InstanceResource):

    class BpType(object):
        READY = "ready"
        ACTIVE = "active"

    def __init__(self, version, payload, sim_sid):
        """
        Initialize the BillingPeriodInstance

        :returns: twilio.rest.supersim.v1.sim.billing_period.BillingPeriodInstance
        :rtype: twilio.rest.supersim.v1.sim.billing_period.BillingPeriodInstance
        """
        super(BillingPeriodInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'account_sid': payload.get('account_sid'),
            'sim_sid': payload.get('sim_sid'),
            'start_time': deserialize.iso8601_datetime(payload.get('start_time')),
            'end_time': deserialize.iso8601_datetime(payload.get('end_time')),
            'period_type': payload.get('period_type'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
        }

        # Context
        self._context = None
        self._solution = {'sim_sid': sim_sid, }

    @property
    def sid(self):
        """
        :returns: The SID of the Billing Period
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account the Super SIM belongs to
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def sim_sid(self):
        """
        :returns: The SID of the Super SIM the Billing Period belongs to
        :rtype: unicode
        """
        return self._properties['sim_sid']

    @property
    def start_time(self):
        """
        :returns: The start time of the Billing Period
        :rtype: datetime
        """
        return self._properties['start_time']

    @property
    def end_time(self):
        """
        :returns: The end time of the Billing Period
        :rtype: datetime
        """
        return self._properties['end_time']

    @property
    def period_type(self):
        """
        :returns: The type of the Billing Period
        :rtype: BillingPeriodInstance.BpType
        """
        return self._properties['period_type']

    @property
    def date_created(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was last updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Supersim.V1.BillingPeriodInstance>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import serialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class AssistantInitiationActionsList(ListResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, assistant_sid):
        """
        Initialize the AssistantInitiationActionsList

        :param Version version: Version that contains the resource
        :param assistant_sid: The assistant_sid

        :returns: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsList
        :rtype: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsList
        """
        super(AssistantInitiationActionsList, self).__init__(version)

        # Path Solution
        self._solution = {'assistant_sid': assistant_sid, }

    def get(self):
        """
        Constructs a AssistantInitiationActionsContext

        :returns: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsContext
        :rtype: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsContext
        """
        return AssistantInitiationActionsContext(
            self._version,
            assistant_sid=self._solution['assistant_sid'],
        )

    def __call__(self):
        """
        Constructs a AssistantInitiationActionsContext

        :returns: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsContext
        :rtype: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsContext
        """
        return AssistantInitiationActionsContext(
            self._version,
            assistant_sid=self._solution['assistant_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Preview.Understand.AssistantInitiationActionsList>'


class AssistantInitiationActionsPage(Page):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, response, solution):
        """
        Initialize the AssistantInitiationActionsPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param assistant_sid: The assistant_sid

        :returns: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsPage
        :rtype: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsPage
        """
        super(AssistantInitiationActionsPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of AssistantInitiationActionsInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsInstance
        :rtype: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsInstance
        """
        return AssistantInitiationActionsInstance(
            self._version,
            payload,
            assistant_sid=self._solution['assistant_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Preview.Understand.AssistantInitiationActionsPage>'


class AssistantInitiationActionsContext(InstanceContext):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, assistant_sid):
        """
        Initialize the AssistantInitiationActionsContext

        :param Version version: Version that contains the resource
        :param assistant_sid: The assistant_sid

        :returns: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsContext
        :rtype: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsContext
        """
        super(AssistantInitiationActionsContext, self).__init__(version)

        # Path Solution
        self._solution = {'assistant_sid': assistant_sid, }
        self._uri = '/Assistants/{assistant_sid}/InitiationActions'.format(**self._solution)

    def fetch(self):
        """
        Fetch the AssistantInitiationActionsInstance

        :returns: The fetched AssistantInitiationActionsInstance
        :rtype: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return AssistantInitiationActionsInstance(
            self._version,
            payload,
            assistant_sid=self._solution['assistant_sid'],
        )

    def update(self, initiation_actions=values.unset):
        """
        Update the AssistantInitiationActionsInstance

        :param dict initiation_actions: The initiation_actions

        :returns: The updated AssistantInitiationActionsInstance
        :rtype: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsInstance
        """
        data = values.of({'InitiationActions': serialize.object(initiation_actions), })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return AssistantInitiationActionsInstance(
            self._version,
            payload,
            assistant_sid=self._solution['assistant_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Preview.Understand.AssistantInitiationActionsContext {}>'.format(context)


class AssistantInitiationActionsInstance(InstanceResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, payload, assistant_sid):
        """
        Initialize the AssistantInitiationActionsInstance

        :returns: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsInstance
        :rtype: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsInstance
        """
        super(AssistantInitiationActionsInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'assistant_sid': payload.get('assistant_sid'),
            'url': payload.get('url'),
            'data': payload.get('data'),
        }

        # Context
        self._context = None
        self._solution = {'assistant_sid': assistant_sid, }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: AssistantInitiationActionsContext for this AssistantInitiationActionsInstance
        :rtype: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsContext
        """
        if self._context is None:
            self._context = AssistantInitiationActionsContext(
                self._version,
                assistant_sid=self._solution['assistant_sid'],
            )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The account_sid
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def assistant_sid(self):
        """
        :returns: The assistant_sid
        :rtype: unicode
        """
        return self._properties['assistant_sid']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def data(self):
        """
        :returns: The data
        :rtype: dict
        """
        return self._properties['data']

    def fetch(self):
        """
        Fetch the AssistantInitiationActionsInstance

        :returns: The fetched AssistantInitiationActionsInstance
        :rtype: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsInstance
        """
        return self._proxy.fetch()

    def update(self, initiation_actions=values.unset):
        """
        Update the AssistantInitiationActionsInstance

        :param dict initiation_actions: The initiation_actions

        :returns: The updated AssistantInitiationActionsInstance
        :rtype: twilio.rest.preview.understand.assistant.assistant_initiation_actions.AssistantInitiationActionsInstance
        """
        return self._proxy.update(initiation_actions=initiation_actions, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Preview.Understand.AssistantInitiationActionsInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class WebhookList(ListResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, assistant_sid):
        """
        Initialize the WebhookList

        :param Version version: Version that contains the resource
        :param assistant_sid: The SID of the Assistant that is the parent of the resource

        :returns: twilio.rest.autopilot.v1.assistant.webhook.WebhookList
        :rtype: twilio.rest.autopilot.v1.assistant.webhook.WebhookList
        """
        super(WebhookList, self).__init__(version)

        # Path Solution
        self._solution = {'assistant_sid': assistant_sid, }
        self._uri = '/Assistants/{assistant_sid}/Webhooks'.format(**self._solution)

    def stream(self, limit=None, page_size=None):
        """
        Streams WebhookInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.autopilot.v1.assistant.webhook.WebhookInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, limit=None, page_size=None):
        """
        Lists WebhookInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.autopilot.v1.assistant.webhook.WebhookInstance]
        """
        return list(self.stream(limit=limit, page_size=page_size, ))

    def page(self, page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of WebhookInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of WebhookInstance
        :rtype: twilio.rest.autopilot.v1.assistant.webhook.WebhookPage
        """
        data = values.of({'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return WebhookPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of WebhookInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of WebhookInstance
        :rtype: twilio.rest.autopilot.v1.assistant.webhook.WebhookPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return WebhookPage(self._version, response, self._solution)

    def create(self, unique_name, events, webhook_url, webhook_method=values.unset):
        """
        Create the WebhookInstance

        :param unicode unique_name: An application-defined string that uniquely identifies the resource
        :param unicode events: The list of space-separated events that this Webhook will subscribe to.
        :param unicode webhook_url: The URL associated with this Webhook.
        :param unicode webhook_method: The method to be used when calling the webhook's URL.

        :returns: The created WebhookInstance
        :rtype: twilio.rest.autopilot.v1.assistant.webhook.WebhookInstance
        """
        data = values.of({
            'UniqueName': unique_name,
            'Events': events,
            'WebhookUrl': webhook_url,
            'WebhookMethod': webhook_method,
        })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return WebhookInstance(self._version, payload, assistant_sid=self._solution['assistant_sid'], )

    def get(self, sid):
        """
        Constructs a WebhookContext

        :param sid: The unique string that identifies the resource to fetch

        :returns: twilio.rest.autopilot.v1.assistant.webhook.WebhookContext
        :rtype: twilio.rest.autopilot.v1.assistant.webhook.WebhookContext
        """
        return WebhookContext(self._version, assistant_sid=self._solution['assistant_sid'], sid=sid, )

    def __call__(self, sid):
        """
        Constructs a WebhookContext

        :param sid: The unique string that identifies the resource to fetch

        :returns: twilio.rest.autopilot.v1.assistant.webhook.WebhookContext
        :rtype: twilio.rest.autopilot.v1.assistant.webhook.WebhookContext
        """
        return WebhookContext(self._version, assistant_sid=self._solution['assistant_sid'], sid=sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Autopilot.V1.WebhookList>'


class WebhookPage(Page):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, response, solution):
        """
        Initialize the WebhookPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param assistant_sid: The SID of the Assistant that is the parent of the resource

        :returns: twilio.rest.autopilot.v1.assistant.webhook.WebhookPage
        :rtype: twilio.rest.autopilot.v1.assistant.webhook.WebhookPage
        """
        super(WebhookPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of WebhookInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.autopilot.v1.assistant.webhook.WebhookInstance
        :rtype: twilio.rest.autopilot.v1.assistant.webhook.WebhookInstance
        """
        return WebhookInstance(self._version, payload, assistant_sid=self._solution['assistant_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Autopilot.V1.WebhookPage>'


class WebhookContext(InstanceContext):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, assistant_sid, sid):
        """
        Initialize the WebhookContext

        :param Version version: Version that contains the resource
        :param assistant_sid: The SID of the Assistant that is the parent of the resource to fetch
        :param sid: The unique string that identifies the resource to fetch

        :returns: twilio.rest.autopilot.v1.assistant.webhook.WebhookContext
        :rtype: twilio.rest.autopilot.v1.assistant.webhook.WebhookContext
        """
        super(WebhookContext, self).__init__(version)

        # Path Solution
        self._solution = {'assistant_sid': assistant_sid, 'sid': sid, }
        self._uri = '/Assistants/{assistant_sid}/Webhooks/{sid}'.format(**self._solution)

    def fetch(self):
        """
        Fetch the WebhookInstance

        :returns: The fetched WebhookInstance
        :rtype: twilio.rest.autopilot.v1.assistant.webhook.WebhookInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return WebhookInstance(
            self._version,
            payload,
            assistant_sid=self._solution['assistant_sid'],
            sid=self._solution['sid'],
        )

    def update(self, unique_name=values.unset, events=values.unset,
               webhook_url=values.unset, webhook_method=values.unset):
        """
        Update the WebhookInstance

        :param unicode unique_name: An application-defined string that uniquely identifies the resource
        :param unicode events: The list of space-separated events that this Webhook will subscribe to.
        :param unicode webhook_url: The URL associated with this Webhook.
        :param unicode webhook_method: The method to be used when calling the webhook's URL.

        :returns: The updated WebhookInstance
        :rtype: twilio.rest.autopilot.v1.assistant.webhook.WebhookInstance
        """
        data = values.of({
            'UniqueName': unique_name,
            'Events': events,
            'WebhookUrl': webhook_url,
            'WebhookMethod': webhook_method,
        })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return WebhookInstance(
            self._version,
            payload,
            assistant_sid=self._solution['assistant_sid'],
            sid=self._solution['sid'],
        )

    def delete(self):
        """
        Deletes the WebhookInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Autopilot.V1.WebhookContext {}>'.format(context)


class WebhookInstance(InstanceResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, payload, assistant_sid, sid=None):
        """
        Initialize the WebhookInstance

        :returns: twilio.rest.autopilot.v1.assistant.webhook.WebhookInstance
        :rtype: twilio.rest.autopilot.v1.assistant.webhook.WebhookInstance
        """
        super(WebhookInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'url': payload.get('url'),
            'account_sid': payload.get('account_sid'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'assistant_sid': payload.get('assistant_sid'),
            'sid': payload.get('sid'),
            'unique_name': payload.get('unique_name'),
            'events': payload.get('events'),
            'webhook_url': payload.get('webhook_url'),
            'webhook_method': payload.get('webhook_method'),
        }

        # Context
        self._context = None
        self._solution = {'assistant_sid': assistant_sid, 'sid': sid or self._properties['sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: WebhookContext for this WebhookInstance
        :rtype: twilio.rest.autopilot.v1.assistant.webhook.WebhookContext
        """
        if self._context is None:
            self._context = WebhookContext(
                self._version,
                assistant_sid=self._solution['assistant_sid'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def url(self):
        """
        :returns: The absolute URL of the Webhook resource
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the resource
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def date_created(self):
        """
        :returns: The RFC 2822 date and time in GMT when the resource was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The RFC 2822 date and time in GMT when the resource was last updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def assistant_sid(self):
        """
        :returns: The SID of the Assistant that is the parent of the resource
        :rtype: unicode
        """
        return self._properties['assistant_sid']

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def unique_name(self):
        """
        :returns: An application-defined string that uniquely identifies the resource
        :rtype: unicode
        """
        return self._properties['unique_name']

    @property
    def events(self):
        """
        :returns: The list of space-separated events that this Webhook is subscribed to.
        :rtype: unicode
        """
        return self._properties['events']

    @property
    def webhook_url(self):
        """
        :returns: The URL associated with this Webhook.
        :rtype: unicode
        """
        return self._properties['webhook_url']

    @property
    def webhook_method(self):
        """
        :returns: The method used when calling the webhook's URL.
        :rtype: unicode
        """
        return self._properties['webhook_method']

    def fetch(self):
        """
        Fetch the WebhookInstance

        :returns: The fetched WebhookInstance
        :rtype: twilio.rest.autopilot.v1.assistant.webhook.WebhookInstance
        """
        return self._proxy.fetch()

    def update(self, unique_name=values.unset, events=values.unset,
               webhook_url=values.unset, webhook_method=values.unset):
        """
        Update the WebhookInstance

        :param unicode unique_name: An application-defined string that uniquely identifies the resource
        :param unicode events: The list of space-separated events that this Webhook will subscribe to.
        :param unicode webhook_url: The URL associated with this Webhook.
        :param unicode webhook_method: The method to be used when calling the webhook's URL.

        :returns: The updated WebhookInstance
        :rtype: twilio.rest.autopilot.v1.assistant.webhook.WebhookInstance
        """
        return self._proxy.update(
            unique_name=unique_name,
            events=events,
            webhook_url=webhook_url,
            webhook_method=webhook_method,
        )

    def delete(self):
        """
        Deletes the WebhookInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Autopilot.V1.WebhookInstance {}>'.format(context)
